# the plugin code was generated by this script
# you should not change the plugin code directly, but this script
# note: this script only creates objects in your workspace,
# *EXCEPT* for the last call, see below.

local({
# set the output directory to overwrite the actual plugin
output.dir <- tempdir()
overwrite <- TRUE

require(rkwarddev)

about.info <- rk.XML.about(
	name="rk.FactorAnalysis",
	author=c(
		person(given="Meik", family="Michalke",
			email="meik.michalke@hhu.de", role=c("aut","cre"))),
	about=list(desc="RKWard GUI to conduct principal component and factor analysis",
		version="0.01-3", url="http://rkward.sf.net", long.desc="RKWard GUI to conduct principal component and factor analysis (using the psych package). Also includes dialogs for scree plots and correlation plots."),
	dependencies=list(rkward.min="0.5.6"),
	package=list(c(name="psych", min="1.1.10"))
	)

#############
## principal component & factor analysis
#############
# using principal()/fa() of the package 'psych'

radio.analysis <- rk.XML.radio("Factoring method", options=list(
		"Principal component analysis"=c(val="PCA"),
		"Explorative factor analysis"=c(val="EFA", chk=TRUE)
	))
var.select <- rk.XML.varselector(label="Select data")
var.data <- rk.XML.varslot(label="Correlation matrix (or raw data matrix)", source=var.select, required=TRUE)

save.results <- rk.XML.saveobj("Save results to workspace", initial="analysis.results")

drp.rotation.PCA <- rk.XML.dropdown("Rotation method", options=list(
		"none"=c(val="none"),
		"varimax (orthogonal)"=c(val="varimax", chk=TRUE),
		"quatimax (orthogonal)"=c(val="quatimax"),
		"promax"=c(val="promax"),
		"oblimin"=c(val="oblimin"),
		"simplimax"=c(val="simplimax"),
		"cluster"=c(val="cluster")
	), id.name="drp_PCA_rotate")
drp.rotation.EFA <- rk.XML.dropdown("Rotation method", options=list(
		"None"=c(val="none"),
		"Varimax (orthogonal)"=c(val="varimax"),
		"Quatimax (orthogonal)"=c(val="quatimax"),
		"BentlerT (orthogonal)"=c(val="bentlerT"),
		"GeominT (orthogonal)"=c(val="geominT"),
		"Bifactor (orthogonal)"=c(val="bifactor"),
		"Promax"=c(val="promax"),
		"Oblimin"=c(val="oblimin", chk=TRUE),
		"Simplimax"=c(val="simplimax"),
		"BentlerQ"=c(val="bentlerQ"),
		"GeominQ"=c(val="geominQ"),
		"Biquartimin"=c(val="biquartimin"),
		"Cluster"=c(val="cluster")
	), id.name="drp_EFA_rotate")

drp.factmeth.EFA <- rk.XML.dropdown("Factoring method", options=list(
		"Minimum residual (OLS)"=c(val="minres", chk=TRUE),
		"Weighted least squares (WLS)"=c(val="wls"),
		"Generalized weighted least squares (GLS)"=c(val="gls"),
		"Principal axis"=c(val="pa"),
		"Maximum likelihood"=c(val="ml")
	), id.name="drp_EFA_factmeth")

radio.SMC.EFA <- rk.XML.radio("Initial communality estimate", options=list(
		"Squared multiple correlations"=c(val="true", chk=TRUE),
		"1"=c(val="false")
	))
radio.covar.EFA <- rk.XML.radio("Matrix to factor", options=list(
		"Factor correlation matrix"=c(val="false", chk=TRUE),
		"Factor covariance matrix"=c(val="true")
	))
radio.scrmtx.EFA <- rk.XML.radio("Matrix to score", options=list(
		"Scores based on structure matrix (oblique)"=c(val="true", chk=TRUE),
		"Scores based on pattern matrix"=c(val="false")
	))

# common options
spin.nfactors <- rk.XML.spinbox("Number of factors to extract", min=1, initial=1, real=FALSE)

chk.residuals <- rk.XML.cbox("Show residuals", value="true")
drp.scores.EFA <- rk.XML.dropdown("Method to find factor scores", options=list(
	"Regression"=c(val="regression", chk=TRUE),
	"Simple regression (Thurstone)"=c(val="Thurstone"),
	"correlaton preserving (ten Berge)"=c(val="tenBerge"),
	"Anderson"=c(val="Anderson"),
	"Bartlett"=c(val="Bartlett")))
drp.missings.PCA <- rk.XML.dropdown("Dealing with missing values", options=list(
	"Don't impute missing values"=c(val="none", chk=TRUE),
	"Replace with median"=c(val="median"),
	"Replace with mean"=c(val="mean")), id.name="radio_missings_PCA")
drp.missings.EFA <- rk.XML.dropdown("Dealing with missing values", options=list(
	"Don't impute missing values"=c(val="none", chk=TRUE),
	"Replace with median"=c(val="median"),
	"Replace with mean"=c(val="mean")), id.name="radio_missings_EFA")
chk.scores <- rk.XML.frame(drp.missings.PCA, label="Find component scores", checkable=TRUE, chk=FALSE)
spin.nobs <- rk.XML.spinbox("Number of observations to find the correlation matrix (GoF statistics)", min=0, initial=0, real=FALSE)

spin.niter <- rk.XML.spinbox("Number of iterations", min=2, initial=2, real=FALSE)
spin.minerr <- rk.XML.spinbox("Until change in communalities is less than", min=0.0001, initial=0.001)
spin.maxiter <- rk.XML.spinbox("Maximum number of iterations", min=2, initial=50, real=FALSE)
chk.iterate <- rk.XML.frame(
	rk.XML.row(
		rk.XML.col(spin.niter),
		rk.XML.col(spin.minerr),
		rk.XML.col(spin.maxiter)
	),
	label="Perform bootstrap iterations", checkable=TRUE, chk=FALSE)

tab1.data <- rk.XML.row(
		var.select,
		rk.XML.col(
			rk.XML.frame(radio.analysis),
			rk.XML.frame(var.data),
			spin.nfactors,
			rk.XML.stretch(),
			save.results
		)
	)

tab2.options <- rk.XML.row(
		rk.XML.col(
			rk.XML.row(
				rk.XML.col(
					drp.rotation.PCA,
					drp.rotation.EFA
				),
				rk.XML.col(
					drp.factmeth.EFA
				)
			),
			rk.XML.row(
				rk.XML.col(radio.SMC.EFA),
				rk.XML.col(radio.covar.EFA),
				rk.XML.col(radio.scrmtx.EFA)
			),
			rk.XML.frame(
				rk.XML.row(
					rk.XML.col(chk.residuals),
					rk.XML.col()
				)
			),
			chk.scores,
			rk.XML.row(
				rk.XML.col(drp.scores.EFA),
				rk.XML.col(drp.missings.EFA)
			),
 			spin.nobs,
			chk.iterate,
 			rk.XML.stretch()
 		)
 	)

full.dialog <- rk.XML.dialog(rk.XML.tabbook(label="Factor analysis",
		tabs=list("Data"=tab1.data, "Options"=tab2.options)
	), label="Factor analysis")

## logic section
	lgc.sect <- rk.XML.logic(
		FA.gov.between <- rk.XML.convert(sources=list(string=radio.analysis), mode=c(equals="PCA")),
		rk.XML.connect(governor=FA.gov.between, client=drp.rotation.PCA, set="visible"),
		rk.XML.connect(governor=FA.gov.between, client=drp.rotation.EFA, set="visible", not=TRUE),
		rk.XML.connect(governor=FA.gov.between, client=drp.factmeth.EFA, set="enabled", not=TRUE),
		rk.XML.connect(governor=FA.gov.between, client=radio.SMC.EFA, set="enabled", not=TRUE),
		rk.XML.connect(governor=FA.gov.between, client=radio.covar.EFA, set="enabled", not=TRUE),
		rk.XML.connect(governor=FA.gov.between, client=radio.scrmtx.EFA, set="enabled", not=TRUE),
		rk.XML.connect(governor=FA.gov.between, client=chk.scores, set="visible"),
		rk.XML.connect(governor=FA.gov.between, client=drp.scores.EFA, set="visible", not=TRUE),
		rk.XML.connect(governor=FA.gov.between, client=drp.missings.EFA, set="visible", not=TRUE),
		rk.XML.connect(governor=FA.gov.between, client=chk.iterate, set="enabled", not=TRUE)
)

## JavaScript
# discard this objest later, we just need the name...
js.frm.score <- rk.JS.vars(chk.scores, modifiers="checked")
js.frm.iterate <- rk.JS.vars(chk.iterate, modifiers="checked")
js.calc <- rk.paste.JS(
	echo("analysis.results <- "),
	ite(id(radio.analysis, " == \"PCA\""),
		echo("principal("),
		echo("fa(")),
	ite(var.data, echo("r=", var.data)),
	ite(id(spin.nfactors, " > 1"), echo(",\n\tnfactors=", spin.nfactors)),
	ite(chk.residuals, echo(",\n\tresiduals=TRUE")),
	ite(id(radio.analysis, " == \"PCA\""),
		echo(",\n\trotate=\"", drp.rotation.PCA, "\""),
		echo(",\n\trotate=\"", drp.rotation.EFA, "\"")),
	ite(id(spin.nobs, " > 0"), echo(",\n\tn.obs=", spin.nobs)),
	ite(id(radio.analysis, " == \"PCA\""),
		rk.paste.JS(
			ite(js.frm.score, echo(",\n\tscores=TRUE")),
			ite(id(js.frm.score, " & ", drp.missings.PCA, " != \"none\""), echo(",\n\tmissing=TRUE,\n\timpute=\"", drp.missings.PCA, "\"")),
		level=3),
		rk.paste.JS(
			ite(id(js.frm.iterate), echo(",\n\tn.iter=", spin.niter)),
			echo(",\n\tscores=\"", drp.scores.EFA, "\""),
			ite(id(radio.SMC.EFA, " == \"false\"" ), echo(",\n\tSMC=FALSE")),
			ite(id(radio.covar.EFA, " == \"true\"" ), echo(",\n\tcovar=TRUE")),
			ite(id(drp.missings.EFA, " != \"none\""), echo(",\n\tmissing=TRUE,\n\timpute=\"", drp.missings.EFA, "\"")),
			ite(id(js.frm.iterate, " & ", spin.minerr, " != 0.001"), echo(",\n\tmin.err=", spin.minerr)),
			ite(id(js.frm.iterate, " & ", spin.maxiter, " != 50"), echo(",\n\tmax.iter=", spin.maxiter)),
			echo(",\n\tfm=\"", drp.factmeth.EFA, "\""),
			ite(id(radio.scrmtx.EFA, " == \"false\"" ), echo(",\n\toblique.scores=FALSE")),
		level=3)),
	echo(")\n\n")
)

js.print <- rk.paste.JS(
 	echo("rk.print(analysis.results[[\"loadings\"]], cutoff=NULL)\n")
)

############
## scree plot
############
scree.var.select <- rk.XML.varselector(label="Select data.frame")
scree.var.data <- rk.XML.varslot(label="Data", source=var.select, required=TRUE)

scree.radio.crv <- rk.XML.radio(label="Draw scree for", options=list(
		"Factors and components"=c(val="both"),
		"Factors only"=c(val="fact"),
		"Prinicipal components only"=c(val="comp")
	))

scree.main <- rk.XML.input(label="Main title", initial="Scree plot")

scree.hline.frm <- rk.XML.frame(
	scree.hline <- rk.XML.spinbox(label="Eigenvalue", min=0, initial=1),
		checkable=TRUE, chk=TRUE, label="Horizontal line")

scree.preview <- rk.XML.preview()

scree.full.dialog <- rk.XML.dialog(
	rk.XML.row(
		scree.var.select,
		rk.XML.col(
			scree.var.data,
			scree.main,
			scree.radio.crv,
			scree.hline.frm,
			rk.XML.stretch(),
			scree.preview
# 			rk.XML.frame(var.dv, var.wid),
# 			rk.XML.frame(var.within, var.between)
		)
	)
, label="Scree plot")

## JavaScript
scree.js.print <- rk.paste.JS(
	scree.chk.hline <- rk.JS.vars(scree.hline.frm, modifiers="checked"),
	ite("full", echo("rk.graph.on()\n")),
	echo("\ttry({\n\t\tscree("),
	ite(scree.var.data, echo("\n\t\t\t", scree.var.data)),
	ite(id(scree.radio.crv, " == \"comp\""), echo(",\n\t\t\tfactors=FALSE")),
	ite(id(scree.radio.crv, " == \"fact\""), echo(",\n\t\t\tpc=FALSE")),
	ite(id(scree.main, " != \"Scree plot\""), echo(",\n\t\t\tmain=\"", scree.main, "\"")),
	ite(scree.chk.hline,
		rk.paste.JS(ite(id(scree.hline, " != 1"), echo(",\n\t\t\thline=", scree.hline)), level=1),
		echo(",\n\t\t\thline=-1")),
	echo(")\n\t})\n"),
 	ite("full", echo("rk.graph.off()\n"))
)

## make a whole component
scree.component <- rk.plugin.component("Scree plot",
	xml=list(
		dialog=scree.full.dialog),
	js=list(
		require="psych",
 		doPrintout=scree.js.print),
	hierarchy=list("plots", "Factor analysis"),
	create=c("xml", "js"))


############
## correlation plot
############
crplt.var.select <- rk.XML.varselector(label="Select data")
crplt.var.data <- rk.XML.varslot(label="Data (correaltaion/factor matrix)",
	source=crplt.var.select,
#	classes=c("data.frame", "matrix"),
	required=TRUE)

crplt.main <- rk.XML.input(label="Main title", initial="Correlation plot")

crplt.radio.colors <- rk.XML.radio(label="Colors", options=list(
		"Red to Blue"=c(val="true", chk=TRUE),
		"Greyscale"=c(val="false")
	))
crplt.spin.shades <- rk.XML.spinbox(label="Number of shades", min=2, initial=51, real=FALSE)

crplt.spin.lower <- rk.XML.spinbox(label="from", min=-1, max=1, initial=-1)
crplt.spin.upper <- rk.XML.spinbox(label="to", min=-1, max=1, initial=1)
crplt.range <- rk.XML.frame(rk.XML.row(rk.XML.col(crplt.spin.lower),rk.XML.col(crplt.spin.upper)), label="Range of correlation values to color")

crplt.spin.nlabels <- rk.XML.spinbox(label="Number of categories in legend", min=1, initial=10, real=FALSE)

crplt.preview <- rk.XML.preview()

crplt.full.dialog <- rk.XML.dialog(
	rk.XML.row(
		crplt.var.select,
		rk.XML.col(
			crplt.var.data,
			crplt.radio.colors,
			crplt.range,
			crplt.spin.shades,
			rk.XML.stretch(),
			crplt.main,
			crplt.chk.lables <- rk.XML.frame(crplt.spin.nlabels,
				checkable=TRUE, chk=TRUE, label="Show legend"),
			crplt.preview
		)
	)
, label="Correlation plot")

## JavaScript
crplt.js.print <- rk.paste.JS(
	js.chk.lables <- rk.JS.vars(crplt.chk.lables, modifiers="checked"),
	ite("full", echo("rk.graph.on()\n")),
	echo("\ttry({\n\t\tcor.plot("),
	ite(crplt.var.data, echo("\n\t\t\tr=", crplt.var.data)),
	ite(id(crplt.radio.colors, " == \"false\""), echo(",\n\t\t\tcolors=FALSE")),
	ite(id(crplt.spin.shades, " != 51"), echo(",\n\t\t\tn=",crplt.spin.shades)),
	ite(id(crplt.main, " != \"Correlation plot\""), echo(",\n\t\t\tmain=\"", crplt.main, "\"")),
	ite(id(crplt.spin.lower, " != -1 | ", crplt.spin.upper, " != 1"),
		echo(",\n\t\t\tzlim=c(",crplt.spin.lower,",",crplt.spin.upper, ")")),
	ite(js.chk.lables,
		rk.paste.JS(
			ite(id(crplt.spin.nlabels, " != 10"), echo(",\n\t\t\tn.legend=", crplt.spin.nlabels)),
			level=1),
		echo(",\n\t\t\tshow.legend=FALSE")),
	echo("\n\t\t)\n\t})\n"),
	ite("full", echo("rk.graph.off()\n"))
)

## make a whole component
crplt.component <- rk.plugin.component("Correlation plot",
	xml=list(
		dialog=crplt.full.dialog),
	js=list(results.header="Correlation plot",
		require="psych",
 		doPrintout=crplt.js.print),
	hierarchy=list("plots", "Factor analysis"),
	create=c("xml", "js"))


#############
## if you run the following function call, files will be written to tempdir!
#############
# this is where it get's serious, that is, here all of the above is put together into one plugin

rk.FactorAnalysis.dir <<- rk.plugin.skeleton(
	about.info,
	path=output.dir,
	xml=list(
		logic=lgc.sect,
		dialog=full.dialog),
	js=list(
		require="psych",
		calculate=js.calc,
		printout=js.print),
	pluginmap=list(name="Factor analysis", hierarchy=list("analysis", "Factor analysis")),
	components=list(scree.component, crplt.component),
	create=c("pmap", "xml", "js", "desc"),
	overwrite=overwrite,
	tests=FALSE,
#	edit=TRUE,
#	show=TRUE,
	load=TRUE)
})
